[libvirt] [PATCH 1/2] util: guarantee sane errno in virFileIsExecutable

Eric Blake eblake at redhat.com
Fri Mar 18 20:46:49 UTC 2011


If virFileIsExecutable is to replace access(file,X_OK), then
errno must be usable on failure.

* src/util/util.c (virFileIsExecutable): Set errno on failure.
---

 src/util/util.c |   12 ++++++++----
 1 files changed, 8 insertions(+), 4 deletions(-)

diff --git a/src/util/util.c b/src/util/util.c
index 5b5dd5e..1e4e2ab 100644
--- a/src/util/util.c
+++ b/src/util/util.c
@@ -1357,7 +1357,8 @@ bool virFileExists(const char *path)
     return access(path, F_OK) == 0;
 }

 /* Check that a file is regular and has executable bits.
  *
  * Note: In the presence of ACLs, this may return true for a file that
  * would actually fail with EACCES for a given user, or false for a
@@ -1370,9 +1371,12 @@ virFileIsExecutable(const char *file)

     /* We would also want to check faccessat if we cared about ACLs,
      * but we don't.  */
-    return (stat(file, &sb) == 0 &&
-            S_ISREG(sb.st_mode) &&
-            (sb.st_mode & 0111) != 0);
+    if (stat(file, &sb) < 0)
+        return false;
+    if (S_ISREG(sb.st_mode) && (sb.st_mode & 0111) != 0)
+        return true;
+    errno = S_ISDIR(sb.st_mode) ? EISDIR : EACCES;
+    return false;
 }

 #ifndef WIN32
@@ -1357,7 +1357,8 @@ bool virFileExists(const char *path)
 }

-/* Check that a file is regular and has executable bits.
+/* Check that a file is regular and has executable bits.  If false is
+ * returned, errno is valid.
  *
  * Note: In the presence of ACLs, this may return true for a file that
-- 
1.7.4




More information about the libvir-list mailing list